/*
 Copyright (C) 2008  Alejandro Gutierrez Munoz

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include "graph.h"

#include <sstream>
#include <queue>
#include <map>


graph::graph(void){
}

graph::graph(const string &s)
{
	try {

		int n,m;
		char c;
		string str;
		listnode list_node;
		vertex graph_vertice;
		istringstream graph_string(s);
		istringstream buffer;
		string line;

		if (!s.empty()) {

			// get first line and check if it is BEGIN
			getline(graph_string,line);
			buffer.str(line);

			if ( line.compare(0,5,"BEGIN") ) {
				throw (int) __LINE__;
			}

			buffer >> str; // skip the BEGIN
			buffer >> this->graph_name;  // name
			buffer >> this->graph_class;  // class

			buffer.clear();

			getline(graph_string,line);		

			while (!graph_string.eof() && line.compare(0,3,"END")  ) {
													
				buffer.str(line);

				if (line.at(0) == 'v' || line.at(0) == 'V') {

					// read vertice line
					buffer >> c;  //char 'v'
					buffer >> n;   // vertex id
					buffer >> str;  // label

					graph_vertice.id = n-1;
					graph_vertice.label = str;


					// add vertex to the list of vertices at the end of the array
					this->vertices.push_back(graph_vertice); 

				}
				else if (line.at(0) == 'e' || line.at(0) == 'E') {

					// read edge line
					buffer >> c; // char 'e'
					buffer >> n; // "from"  vertex
					buffer >> m; // "to"  vertex
					buffer >> str;  // label

																		
					//check if the size of the list is 0. If so, this means that it's the first
					//edge checked. Proceed to create the list
			
					if ( this->adjacency_list.size() == 0) {
						// create a vector with size |v|.
						this->adjacency_list.resize(this->vertices.size());
					}

					if (n > adjacency_list.size() || m > adjacency_list.size() ) {
						throw (int) __LINE__;
					}
					
					// create list node with id and value
					list_node.id = m-1;
					list_node.value = str;

					this->adjacency_list[n-1].push_back(list_node);
					
				}

				// clear temp variables
				buffer.clear();
				c = '\0';
				n = 0;
				m = 0;
				str = ""; 
				
				// get next line
				getline(graph_string,line);	
				
			}

			if ( line.compare(0,3,"END") ) {
				throw (int) __LINE__;
			}

			graph_string.clear();
		}
		else {
			throw (int) __LINE__;
		}

	}
	catch (int e) {
		cerr << "There was an error loading graph, please check log file" << endl;
		throw e;
	}
}



string graph::exportDOTstring(void) const {

	stringstream temp_str;

	try {

		priority_queue< pair< int,int >,vector< pair< int,int > > > Q; //min-priority queu
		map< pair< int,int >,bool > edge;
		
		temp_str << "graph " << this->graph_class << this->graph_name << "{\n"
				<< "  node[shape=\"circle\"]\n\n";


		// add the list of vertices to a priority queue, this is to
		// add the vertices with more edges to the DOT string first.
		// this will help to obtain a better distplay
		for (int i=0; i < (int) this->vertices.size(); i++ ) {
			Q.push(pair<int,int>(this->adjacency_list[i].size(),i)); 
		}
		
		while (!Q.empty() ) {

			pair<int,int> top = Q.top();
			int i = top.second; 

			temp_str << "  \"v" << i << "\" [label =\"" << i << " " << this->vertices[i].label << "\"]\n";

			Q.pop(); 
		}

		for (int i=0; i < (int) this->vertices.size(); i ++) {  
			for (int j=0; j < (int) this->adjacency_list[i].size(); j++  ) {

					int currentVertex = this->adjacency_list[i][j].id ;
			
					if ( !(edge[pair<int,int>(i,currentVertex)] || edge[pair<int,int>(currentVertex,i)]) ) {
						temp_str << "  \"v" << i << "\"--\"v" << currentVertex << "\" [label =\"" << this->adjacency_list[i][j].value << "\"]\n";
						edge[pair<int,int>(i,currentVertex)] = 1; 
						edge[pair<int,int>(currentVertex,i)] = 1;
					}				
				}
		}


		temp_str << "}";
	}
	catch (bad_exception&) {
		cerr << "There was an error on graph::exportDOTstring() line" << __LINE__ << endl;
	}

	
	return temp_str.str();
}

graph::~graph(void) {
}
